package cn.iocoder.yudao.module.system.controller.admin.sms;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.spring.enums.UserTypeEnum;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.system.controller.admin.sms.vo.smsNode.*;
import cn.iocoder.yudao.module.system.convert.sms.SmsNodeConvert;
import cn.iocoder.yudao.module.system.dal.dataobject.sms.SmsNodeDO;
import cn.iocoder.yudao.module.system.service.sms.SmsNodeService;
import cn.iocoder.yudao.module.system.service.sms.SmsSendService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

@Validated
@RestController
@Api(tags = "管理后台 - 短信节点")
@RequestMapping("/system/sms-node")
public class SmsNodeController {

    @Resource
    private SmsNodeService smsNodeService;

    @Resource
    private SmsSendService smsSendService;

    @PostMapping("/create")
    @ApiOperation("创建短信节点")
    @PreAuthorize("@ss.hasPermission('system:sms-node:create')")
    public CommonResult<Long> createSmsNode(@Valid @RequestBody SmsNodeCreateReqVO createReqVO) {
        return success(smsNodeService.createSmsNode(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新短信节点")
    @PreAuthorize("@ss.hasPermission('system:sms-node:update')")
    public CommonResult<Boolean> updateSmsNode(@Valid @RequestBody SmsNodeUpdateReqVO updateReqVO) {
        smsNodeService.updateSmsNode(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除短信节点")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('system:sms-node:delete')")
    public CommonResult<Boolean> deleteSmsNode(@RequestParam("id") Long id) {
        smsNodeService.deleteSmsNode(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得短信节点")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('system:sms-node:query')")
    public CommonResult<SmsNodeBackVO> getSmsNode(@RequestParam("id") Long id) {
        SmsNodeDO smsNode = smsNodeService.getSmsNode(id);
        SmsNodeBackVO convert = SmsNodeConvert.INSTANCE.convert(smsNode);
        String countryId = convert.getCountryId();
        if (countryId == null || countryId.isEmpty()) {
            convert.setCountryIds(new int[0]);
        } else {
            String[] countryIdsStr = countryId.split(",");
            int[] countryIdsInt = new int[countryIdsStr.length];
            for (int i = 0; i < countryIdsStr.length; i++) {
                try {
                    countryIdsInt[i] = Integer.parseInt(countryIdsStr[i].trim());
                } catch (NumberFormatException e) {
                    throw new IllegalArgumentException("Invalid country ID: " + countryIdsStr[i], e);
                }
            }
            convert.setCountryIds(countryIdsInt);
        }
        String extra = convert.getExtra();
        if (extra != null) {
            SmsNodeExtraVO smsNodeExtraVO = JSON.parseObject(extra, SmsNodeExtraVO.class);
            convert.setReceiveAddrList(smsNodeExtraVO.getReceiveAddrList());
        }
        return success(convert);
    }

    @GetMapping("/list")
    @ApiOperation("获得短信节点列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('system:sms-node:query')")
    public CommonResult<List<SmsNodeBackVO>> getSmsNodeList(@RequestParam("ids") Collection<Long> ids) {
        List<SmsNodeDO> list = smsNodeService.getSmsNodeList(ids);
        return success(SmsNodeConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得短信节点分页")
    @PreAuthorize("@ss.hasPermission('system:sms-node:query')")
    public CommonResult<PageResult<SmsNodeBackVO>> getSmsNodePage(@Valid SmsNodeQueryVO query) {
        PageResult<SmsNodeDO> pageResult = smsNodeService.getSmsNodePage(query);
        return success(SmsNodeConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出短信节点 Excel")
    @PreAuthorize("@ss.hasPermission('system:sms-node:export')")
    @OperateLog(type = EXPORT)
    public void exportSmsNodeExcel(@Valid SmsNodeQueryVO query,
                                   HttpServletResponse response) throws IOException {
        List<SmsNodeDO> list = smsNodeService.getSmsNodeList(query);
        // 导出 Excel
        List<SmsNodeBackVO> datas = SmsNodeConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "短信节点.xls", "数据", SmsNodeBackVO.class, datas);
    }

    @PostMapping("/test")
    @ApiOperation("测试短信节点")
    public CommonResult<Boolean> test(@Valid @RequestBody SmsNodeTestVO smsNodeTestVO) {
        List<Long> longs = smsNodeTestVO.getReceiveAddrList().get(0);
        smsSendService.sendSingleSmsV2(smsNodeTestVO.getMobile(), SecurityFrameworkUtils.getLoginUserId(), UserTypeEnum.ADMIN.getValue(), smsNodeTestVO.getNodeValue(),
                smsNodeTestVO.getCountryCode(), smsNodeTestVO.getIsOrders(), smsNodeTestVO.getIsTransport(), smsNodeTestVO.getTransportId(),
                smsNodeTestVO.getMessageType(), smsNodeTestVO.getTemplateParams(), null, smsNodeTestVO.getNodeTemplateSn(), null, longs.get(0) + Constants.COLON + longs.get(1) + Constants.COLON + longs.get(2));
        return success(true);
    }
}
