package cn.iocoder.yudao.module.system.service.sms;

import cn.iocoder.yudao.framework.apollo.core.constants.CacheConstants;
import cn.iocoder.yudao.framework.apollo.core.constants.Constants;
import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.framework.sms.core.client.SmsClient;
import cn.iocoder.yudao.framework.sms.core.client.SmsClientFactory;
import cn.iocoder.yudao.framework.sms.core.client.dto.SmsLogDTO;
import cn.iocoder.yudao.framework.sms.core.enums.ReceiveStatusEnum;
import cn.iocoder.yudao.module.system.controller.admin.sms.vo.smsNode.SmsNodeCreateReqVO;
import cn.iocoder.yudao.module.system.controller.admin.sms.vo.smsNode.SmsNodeExtraVO;
import cn.iocoder.yudao.module.system.controller.admin.sms.vo.smsNode.SmsNodeQueryVO;
import cn.iocoder.yudao.module.system.controller.admin.sms.vo.smsNode.SmsNodeUpdateReqVO;
import cn.iocoder.yudao.module.system.convert.sms.SmsLogConvert;
import cn.iocoder.yudao.module.system.convert.sms.SmsNodeConvert;
import cn.iocoder.yudao.module.system.dal.dataobject.sms.SmsLogDO;
import cn.iocoder.yudao.module.system.dal.dataobject.sms.SmsNodeDO;
import cn.iocoder.yudao.module.system.dal.mysql.sms.SmsNodeMapper;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.*;

import static cn.iocoder.yudao.framework.apollo.core.constants.CacheConstants.SYSTEM_SMS_NODE_KEY;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.SMS_NODE_NOT_EXISTS;

/**
 * 短信节点 Service 实现类
 *
 * @author jayden
 */
@Slf4j
@Service
@Validated
public class SmsNodeServiceImpl extends AbstractService<SmsNodeMapper, SmsNodeDO> implements SmsNodeService {

    @Resource
    private SmsNodeMapper smsNodeMapper;

    @Resource
    private RedisHelper redisHelper;

    @Resource
    private SmsLogService smsLogService;

    @Resource
    private SmsClientFactory smsClientFactory;

    @Resource
    private SmsSendService smsSendService;

    /**
     * 项目启动时，初始化参数到缓存
     */
    @PostConstruct
    public void init() {
        loadingConfigCache();
    }

    /**
     * 加载参数缓存数据
     * 该方法从数据库中读取短信节点配置，并将它们缓存到Redis中
     * 这样做是为了提高配置数据的读取效率，避免频繁访问数据库
     */
    public void loadingConfigCache() {
        try {
            // 从数据库中选择所有短信节点数据对象
            List<SmsNodeDO> smsNodeDOList = smsNodeMapper.selectList();
            for (SmsNodeDO smsNodeDO : smsNodeDOList) {
                createCache(smsNodeDO);
            }
        } catch (Exception e) {
            // 记录错误日志信息
            log.error("Error loading config cache", e);
        }
    }

    /**
     * 构建缓存键
     * 根据SmsNodeDO对象的属性生成一个唯一的缓存键
     * 这是为了确保每个配置项在缓存中都有一个唯一的标识
     *
     * @param smsNodeDO SmsNodeDO 对象
     * @param receive   目的仓
     * @return 缓存键
     */
    @Override
    public String buildCacheKey(SmsNodeDO smsNodeDO, String receive) {
        // 使用格式化字符串构建缓存键，SYSTEM_SMS_NODE_KEY:短信节点:是否匹配运输方式:运输方式:是否多订单:国家代码:启用状态:目的仓
        return String.format("%s%s:%s:%s:%s:%s:%s:%s",
                SYSTEM_SMS_NODE_KEY,
                smsNodeDO.getNodeValue(),
                smsNodeDO.getIsTransport(),
                smsNodeDO.getTransportId(),
                smsNodeDO.getIsOrders(),
                smsNodeDO.getCountryCode(),
                smsNodeDO.getStatus(),
                receive);
    }

    /**
     * 短信重发定时任务
     */
    public void scheduleReceiveRefresh(Integer time) {
        try {
            // 获取需要重发短信的日志列表
            List<SmsLogDO> smsLogs = smsLogService.selectReceiveList(time);
            if (smsLogs != null) {
                for (SmsLogDO smsLog : smsLogs) {
                    try {
                        processSmsLog(smsLog);
                    } catch (Exception e) {
                        log.error("Error processing sms log: {}", smsLog, e);
                    }
                }
                // 批量更新短信日志
                smsLogService.updateBatchSmsLog(smsLogs);
            }
        } catch (Exception e) {
            log.error("Error in scheduleReceiveRefresh", e);
        }
    }

    /**
     * 处理短信日志
     *
     * @param smsLog 短信日志对象
     */
    private void processSmsLog(SmsLogDO smsLog) {
        // 根据渠道ID获取短信客户端
        SmsClient smsClient = smsClientFactory.getSmsClient(smsLog.getChannelId());
        try {
            // 获取短信接收状态
            SmsLogDTO receiveStatus = smsClient.getReceiveStatus(SmsLogConvert.INSTANCE.toDto(smsLog));
            smsLog.setReceiveStatus(receiveStatus.getReceiveStatus());
        } catch (Exception e) {
            smsLog.setReceiveStatus(ReceiveStatusEnum.Receive_STATUS_20.getValue());
            log.error("获取短信接收状态失败: {}", smsLog, e);
        }

        // 接收失败，则重发
        if (ReceiveStatusEnum.Receive_STATUS_20.getValue().equals(smsLog.getReceiveStatus())) {
            smsLog.setNum(1);
            // 从Redis中获取短信节点信息
            String node = redisHelper.get(SYSTEM_SMS_NODE_KEY + smsLog.getNodeId());
            if (node == null) {
                return;
            }
            // 解析短信节点信息
            SmsNodeDO smsNodeDO = JSON.parseObject(node, SmsNodeDO.class);
            // 调用短信发送服务重新发送短信
            smsSendService.sendSingleSmsV2(smsLog.getMobile(), smsLog.getUserId(), smsLog.getUserType(), smsNodeDO.getNodeValue(),
                    smsNodeDO.getCountryCode(), smsNodeDO.getIsOrders(), smsNodeDO.getIsTransport(), smsNodeDO.getTransportId(),
                    smsLog.getMessageType(), smsLog.getTemplateParams(), smsLog.getId(), smsLog.getNodeTemplateSn() + 1, smsNodeDO, Constants.RECEIVE_ADDR_KEY);
        }
    }

    // 辅助方法：获取目的仓，若为空则初始化目的仓默认值1:1:1
    private List<List<Long>> getReceiveAddrList(SmsNodeExtraVO smsNodeExtraVO) {
        if (smsNodeExtraVO == null || smsNodeExtraVO.getReceiveAddrList() == null || smsNodeExtraVO.getReceiveAddrList().isEmpty()) {
            List<List<Long>> receiveAddrList = new ArrayList<>();
            List<Long> longs = new ArrayList<>();
            longs.add(1L);
            longs.add(1L);
            longs.add(1L);
            receiveAddrList.add(longs);
            return receiveAddrList;
        }
        return smsNodeExtraVO.getReceiveAddrList();
    }

    // 辅助方法：获取目的仓，若为空则初始化目的仓默认值1:1:1
    private List<List<Long>> getReceiveAddrList(List<List<Long>> receiveAddrList) {
        if (receiveAddrList == null || receiveAddrList.isEmpty()) {
            List<List<Long>> list = new ArrayList<>();
            List<Long> longs = new ArrayList<>();
            longs.add(1L);
            longs.add(1L);
            longs.add(1L);
            list.add(longs);
            return list;
        }
        return receiveAddrList;
    }

    // 辅助方法：生成键 节点国家：目的仓国家：目的仓城市：目的仓仓库
    private String generateKey(String countryId, List<Long> integers) {
        if (integers.size() < 3) {
            throw new IllegalArgumentException("Receive address list must have at least 3 elements");
        }
        String targetCountry = String.valueOf(integers.get(0));
        String targetCity = String.valueOf(integers.get(1));
        String receiveAddr = String.valueOf(integers.get(2));
        return countryId + CacheConstants.COLON + targetCountry + CacheConstants.COLON + targetCity + CacheConstants.COLON + receiveAddr;
    }

    /**
     * 创建短信节点
     * <p>
     * 此方法接收一个包含短信节点创建请求的VO对象，验证是否存在重复节点，然后将节点信息插入数据库，
     * 并创建相应的缓存最后返回新创建节点的ID
     *
     * @param createReqVO 短信节点创建请求的VO对象，包含节点的必要信息如节点值、传输ID、排序标志等
     * @return 返回新创建的短信节点的ID
     * @throws ServiceException 如果检测到重复的节点，则抛出服务异常
     */
    @Override
    public Long createSmsNode(SmsNodeCreateReqVO createReqVO) {
        // 将创建请求对象转换为短信节点对象
        SmsNodeDO smsNode = SmsNodeConvert.INSTANCE.convert(createReqVO);
        // 检查是否已存在相同节点
        SmsNodeQueryVO smsNodeDO = new SmsNodeQueryVO();
        smsNodeDO.setNodeValue(createReqVO.getNodeValue());
        smsNodeDO.setTransportId(createReqVO.getTransportId());
        smsNodeDO.setIsOrders(createReqVO.getIsOrders());
        List<SmsNodeDO> smsNodeDOList = smsNodeMapper.selectList(smsNodeDO);
        String country = createReqVO.getCountryId();
        // 获取目的仓
        List<List<Long>> receiveAddrs = getReceiveAddrList(createReqVO.getReceiveAddrList());
        if (smsNodeDOList != null && !smsNodeDOList.isEmpty()) {
            Map<String, Integer> map = new HashMap<>();
            for (SmsNodeDO nodeDO : smsNodeDOList) {
                String countryIds = nodeDO.getCountryId();
                Set<String> existingCountryIds = new HashSet<>(Arrays.asList(countryIds.split(",")));
                SmsNodeExtraVO smsNodeExtraVO = JSON.parseObject(nodeDO.getExtra(), SmsNodeExtraVO.class);
                // 获取目的仓
                List<List<Long>> receiveAddrList = getReceiveAddrList(smsNodeExtraVO);

                for (List<Long> integers : receiveAddrList) {
                    for (String existingCountryId : existingCountryIds) {
                        String countryId = existingCountryId;
                        String targetCountry = String.valueOf(integers.get(0));
                        String targetCity = String.valueOf(integers.get(1));
                        String receiveAddr = String.valueOf(integers.get(2));
                        String key = countryId + CacheConstants.COLON + targetCountry + CacheConstants.COLON + targetCity + CacheConstants.COLON + receiveAddr;
                        map.put(key, 0);
                        if ("0".equals(countryId)) {
                            // 如果存在全部国家，则判断节点是否为单个国家
                            if (!"0".equals(country)) {
                                // 如果节点已存在，抛出异常
                                throw new ServiceException(500, "不能对单个国家设置");
                            }
                        }
                    }
                }
            }
            Set<String> existingCountryIds = new HashSet<>(Arrays.asList(country.split(",")));


            for (List<Long> integers : receiveAddrs) {
                for (String existingCountryId : existingCountryIds) {
                    String key = generateKey(existingCountryId, integers);
                    if (map.containsKey(key)) {
                        // 如果节点已存在，抛出异常
                        throw new ServiceException(500, "不能重复设置");
                    }
                }
            }
        }
        SmsNodeExtraVO smsNodeExtraVO = new SmsNodeExtraVO();
        smsNodeExtraVO.setReceiveAddrList(receiveAddrs);
        smsNode.setExtra(JSON.toJSONString(smsNodeExtraVO));
        // 插入新的短信节点到数据库
        smsNodeMapper.insert(smsNode);
        // 设置新的缓存
        createCache(smsNode);
        // 返回新创建的短信节点的ID
        return smsNode.getId();
    }

    /**
     * 创建缓存
     */
    public void createCache(SmsNodeDO smsNode) {
        String[] countryCodesStr = smsNode.getCountryCode().split(",");
        Map<String, String> maps = new HashMap<>();
        SmsNodeExtraVO smsNodeExtraVO = JSON.parseObject(smsNode.getExtra(), SmsNodeExtraVO.class);
        // 获取目的仓
        List<List<Long>> receiveAddrList = getReceiveAddrList(smsNodeExtraVO);

        for (List<Long> integers : receiveAddrList) {
            for (String s : countryCodesStr) {
                smsNode.setCountryCode(s);
                // 构建缓存键
                String targetCountry = String.valueOf(integers.get(0));
                String targetCity = String.valueOf(integers.get(1));
                String receiveAddr = String.valueOf(integers.get(2));
                String key = buildCacheKey(smsNode, targetCountry + CacheConstants.COLON + targetCity + CacheConstants.COLON + receiveAddr);
                // 将对象转换为JSON字符串作为缓存值
                String value = JSON.toJSONString(smsNode);
                maps.put(key, value);
            }
        }

        // 将键值对存入Redis缓存
        redisHelper.multiSet(maps);
        redisHelper.set(SYSTEM_SMS_NODE_KEY + smsNode.getId(), JSON.toJSONString(smsNode));
    }

    /**
     * 删除缓存
     */
    public void deleteCache(SmsNodeDO smsNode) {
        String[] countryCodesStr = smsNode.getCountryCode().split(",");
        SmsNodeExtraVO smsNodeExtraVO = JSON.parseObject(smsNode.getExtra(), SmsNodeExtraVO.class);
        // 获取目的仓
        List<List<Long>> receiveAddrList = getReceiveAddrList(smsNodeExtraVO);

        Collection<String> keys = new ArrayList<>();
        for (List<Long> integers : receiveAddrList) {
            for (String s : countryCodesStr) {
                smsNode.setCountryCode(s);
                // 构建缓存键
                String targetCountry = String.valueOf(integers.get(0));
                String targetCity = String.valueOf(integers.get(1));
                String receiveAddr = String.valueOf(integers.get(2));
                String key = buildCacheKey(smsNode, targetCountry + CacheConstants.COLON + targetCity + CacheConstants.COLON + receiveAddr);
                // 将对象转换为JSON字符串作为缓存值
                keys.add(key);
            }
        }
        // 将键值对存入Redis缓存
        redisHelper.delete(keys);
        redisHelper.delete(SYSTEM_SMS_NODE_KEY + smsNode.getId());
    }


    /**
     * 更新短信节点信息
     *
     * @param updateReqVO 短信节点更新请求对象，包含需要更新的短信节点信息
     * @throws ServiceException 如果节点重复，则抛出服务异常
     */
    @Override
    public void updateSmsNode(SmsNodeUpdateReqVO updateReqVO) {
        // 校验存在
        SmsNodeDO smsNodeDO = this.validateSmsNodeExists(updateReqVO.getId());
        // 将更新请求对象转换为需要更新的实体对象
        SmsNodeDO updateObj = SmsNodeConvert.INSTANCE.convert(updateReqVO);
        // 从缓存中获取当前节点信息
        SmsNodeQueryVO smsNode = new SmsNodeQueryVO();
        smsNode.setNodeValue(updateReqVO.getNodeValue());
        smsNode.setTransportId(updateReqVO.getTransportId());
        smsNode.setIsOrders(updateReqVO.getIsOrders());

        List<SmsNodeDO> smsNodeDOList = smsNodeMapper.selectList(smsNode);

        // 检查节点是否重复，如果重复且ID不匹配，则抛出异常
        if (smsNodeDOList != null && !smsNodeDOList.isEmpty()) {
            Map<String, SmsNodeDO> map = new HashMap<>();
            String country = updateReqVO.getCountryId();
            for (SmsNodeDO nodeDO : smsNodeDOList) {
                String countryIds = nodeDO.getCountryId();
                Set<String> existingCountryIds = new HashSet<>(Arrays.asList(countryIds.split(",")));
                SmsNodeExtraVO smsNodeExtraVO = JSON.parseObject(nodeDO.getExtra(), SmsNodeExtraVO.class);
                // 获取目的仓
                List<List<Long>> receiveAddrList = getReceiveAddrList(smsNodeExtraVO);

                for (List<Long> integers : receiveAddrList) {
                    for (String existingCountryId : existingCountryIds) {
                        String key = generateKey(existingCountryId, integers);
                        map.put(key, nodeDO);
                        if ("0".equals(existingCountryId)) {
                            if (!"0".equals(country) && nodeDO.getId() != updateObj.getId()) {
                                throw new ServiceException(500, "不能对单个国家设置");
                            }
                        }
                    }
                }
            }

            Set<String> existingCountryIds = new HashSet<>(Arrays.asList(country.split(",")));
            List<List<Long>> receiveAddrList = getReceiveAddrList(updateReqVO.getReceiveAddrList());

            for (List<Long> integers : receiveAddrList) {
                for (String existingCountryId : existingCountryIds) {
                    String key = generateKey(existingCountryId, integers);
                    if (map.containsKey(key)) {
                        SmsNodeDO nodeDO = map.get(key);
                        if (!nodeDO.getId().equals(updateObj.getId())) {
                            // 如果节点已存在，抛出异常
                            throw new ServiceException(500, "不能重复设置");
                        }

                    }
                }
            }
            SmsNodeExtraVO smsNodeExtraVO = new SmsNodeExtraVO();
            smsNodeExtraVO.setReceiveAddrList(receiveAddrList);
            updateObj.setExtra(JSON.toJSONString(smsNodeExtraVO));
        }
        // 更新数据库中的短信节点信息
        smsNodeMapper.updateById(updateObj);
        // 删除旧的缓存
        deleteCache(smsNodeDO);
        // 创建新的缓存
        createCache(updateObj);
    }

    @Override
    public void deleteSmsNode(Long id) {
        // 校验存在
        SmsNodeDO smsNodeDO = this.validateSmsNodeExists(id);
        // 删除
        smsNodeMapper.deleteById(id);
        // 删除缓存中的短信节点信息
        deleteCache(smsNodeDO);
    }

    private SmsNodeDO validateSmsNodeExists(Long id) {
        SmsNodeDO smsNodeDO = smsNodeMapper.selectById(id);
        if (smsNodeDO == null) {
            throw exception(SMS_NODE_NOT_EXISTS);
        }
        return smsNodeDO;
    }

    @Override
    public SmsNodeDO getSmsNode(Long id) {
        return smsNodeMapper.selectById(id);
    }

    @Override
    public List<SmsNodeDO> getSmsNodeList(Collection<Long> ids) {
        return smsNodeMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<SmsNodeDO> getSmsNodePage(SmsNodeQueryVO query) {
        return smsNodeMapper.selectPage(query);
    }

    @Override
    public List<SmsNodeDO> getSmsNodeList(SmsNodeQueryVO query) {
        return smsNodeMapper.selectList(query);
    }

}
