package cn.iocoder.yudao.module.system.service.sms;

import cn.iocoder.yudao.framework.common.validation.Mobile;
import cn.iocoder.yudao.module.system.dal.dataobject.sms.SmsNodeDO;
import cn.iocoder.yudao.module.system.mq.message.sms.SmsSendMessage;
import cn.iocoder.yudao.module.system.mq.message.sms.SmsSendMessageV2;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 短信发送 Service 接口
 *
 * @author 捷道源码
 */
public interface SmsSendService {

    /**
     * 发送单条短信给管理后台的用户
     * <p>
     * 在 mobile 为空时，使用 userId 加载对应管理员的手机号
     *
     * @param mobile         手机号
     * @param userId         用户编号
     * @param templateCode   短信模板编号
     * @param templateParams 短信模板参数
     * @return 发送日志编号
     */
    Long sendSingleSmsToAdmin(String mobile, Long userId, String templateCode,
                              Map<String, Object> templateParams, String areaCode);

    /**
     * 发送短信服务方法
     * 该方法负责根据提供的参数构建短信发送请求，并处理发送逻辑
     *
     * @param mobile         手机号码
     * @param userId         用户ID
     * @param nodeValue      节点值，用于确定短信内容
     * @param areaCode       区域代码，用于确定短信发送的国家或地区
     * @param isOrders       订单相关标志
     * @param isTransport    物流相关标志
     * @param transportId    物流
     * @param messageType    发送类型
     * @param templateParams 短信模板参数
     */
    Long sendSingleSmsToAdminV2(@Mobile @NotEmpty(message = "手机号不能为空") String mobile, Long userId,
                                @NotNull(message = "业务节点不能为空") String nodeValue, @NotNull(message = "国家区号不能为空") String areaCode,
                                @NotNull(message = "是否多订单不能为空") Integer isOrders, @NotNull(message = "是否匹配运输方式不能为空") Integer isTransport,
                                @NotNull(message = "运输方式不能空") Integer transportId, @NotNull(message = "发送类型不能为空") Integer messageType, Map<String, Object> templateParams, String receive);

    /**
     * 发送单条短信给用户 APP 的用户
     * <p>
     * 在 mobile 为空时，使用 userId 加载对应会员的手机号
     *
     * @param mobile         手机号
     * @param userId         用户编号
     * @param templateCode   短信模板编号
     * @param templateParams 短信模板参数
     * @return 发送日志编号
     */
    Long sendSingleSmsToMember(String mobile, Long userId, String templateCode,
                               Map<String, Object> templateParams, String areaCode);


    /**
     * 发送单条短信给用户 APP 的用户
     *
     * @param mobile         手机号码
     * @param userId         用户ID
     * @param nodeValue      节点值，用于确定短信内容
     * @param areaCode       区域代码，用于确定短信发送的国家或地区
     * @param isOrders       订单相关标志
     * @param isTransport    物流相关标志
     * @param transportId    物流
     * @param messageType    发送类型
     * @param templateParams 短信模板参数
     */
    Long sendSingleSmsToMemberV2(@Mobile @NotEmpty(message = "手机号不能为空") String mobile, Long userId,
                                 @NotNull(message = "业务节点不能为空") String nodeValue, @NotNull(message = "国家区号不能为空") String areaCode,
                                 @NotNull(message = "是否多订单不能为空") Integer isOrders, @NotNull(message = "是否匹配运输方式不能为空") Integer isTransport,
                                 @NotNull(message = "运输方式不能空") Integer transportId, @NotNull(message = "发送类型不能为空") Integer messageType, Map<String, Object> templateParams, String receive);

    /**
     * 发送单条短信给用户
     *
     * @param mobile         手机号
     * @param userId         用户编号
     * @param userType       用户类型
     * @param templateCode   短信模板编号
     * @param templateParams 短信模板参数
     * @return 发送日志编号
     */
    Long sendSingleSms(String mobile, Long userId, Integer userType,
                       String templateCode, Map<String, Object> templateParams);


    /**
     * 发送短信服务方法
     * 该方法负责根据提供的参数构建短信发送请求，并处理发送逻辑
     *
     * @param mobile         手机号码
     * @param userId         用户ID
     * @param userType       用户类型
     * @param nodeValue      节点值，用于确定短信内容
     * @param areaCode       区域代码，用于确定短信发送的国家或地区
     * @param isOrders       是否多订单
     * @param isTransport    是否匹配运输方式
     * @param transportId    运输方式
     * @param messageType    发送类型
     * @param templateParams 短信模板参数
     * @param smsLogId       重发短信时的原短信日志id
     * @param nodeTemplateSn 模板序号
     * @param smsNode        节点对象
     * @param receive        目的仓
     */
    Long sendSingleSmsV2(@Mobile @NotEmpty(message = "手机号不能为空") String mobile, Long userId, Integer userType, @NotNull(message = "业务节点不能为空") String nodeValue,
                         @NotNull(message = "国家区号不能为空") String areaCode, @NotNull(message = "是否多订单不能为空") Integer isOrders, @NotNull(message = "是否匹配运输方式不能为空") Integer isTransport,
                         @NotNull(message = "运输方式不能空") Integer transportId, @NotNull(message = "发送类型不能为空") Integer messageType, Map<String, Object> templateParams, Long smsLogId, Integer nodeTemplateSn, SmsNodeDO smsNode,
                         @NotNull(message = "目的仓不能空") String receive);

    void resendSingleSmsBySmsLogIds(Collection<Long> smsLogIds);

    default void sendBatchSms(List<String> mobiles, List<Long> userIds, Integer userType,
                              String templateCode, Map<String, Object> templateParams) {
        throw new UnsupportedOperationException("暂时不支持该操作，感兴趣可以实现该功能哟！");
    }

    /**
     * 执行真正的短信发送
     * 注意，该方法仅仅提供给 MQ Consumer 使用
     *
     * @param message 短信
     */
    void doSendSms(SmsSendMessage message);

    /**
     * 执行真正的短信发送
     * 注意，该方法仅仅提供给 MQ Consumer 使用
     *
     * @param message 短信
     */
    void doSendSmsV2(SmsSendMessageV2 message);

    /**
     * 接收短信的接收结果
     *
     * @param channelCode 渠道编码
     * @param text        结果内容
     * @throws Throwable 处理失败时，抛出异常
     */
    void receiveSmsStatus(String channelCode, String text) throws Throwable;


}
